import { useEffect, useRef, useState } from "react";
import { useThree } from "@react-three/fiber";
import * as THREE from "three";
import { useSubModuleStore } from "../../../../store/useModuleStore";
import { useSelectedAsset } from "../../../../store/simulation/useSimulationStore";
import { useProductStore } from "../../../../store/simulation/useProductStore";
import { useEventsStore } from "../../../../store/simulation/useEventsStore";
import { useSelectedProduct } from "../../../../store/simulation/useSimulationStore";
import { handleAddEventToProduct } from "../../events/points/functions/handleAddEventToProduct";

interface ConnectionLine {
    id: string;
    start: THREE.Vector3;
    end: THREE.Vector3;
    mid: THREE.Vector3;
    trigger: TriggerSchema;
}

function TriggerConnector() {
    const { gl, raycaster, scene } = useThree();
    const { subModule } = useSubModuleStore();
    const { products, getPointByUuid, getIsEventInProduct, getActionByUuid, addTrigger, addEvent, getEventByModelUuid } = useProductStore();
    const { selectedAsset, clearSelectedAsset } = useSelectedAsset();
    const { selectedProduct } = useSelectedProduct();

    const [firstSelectedPoint, setFirstSelectedPoint] = useState<{
        productId: string;
        modelUuid: string;
        pointUuid: string;
        actionUuid?: string;
    } | null>(null);

    const [connections, setConnections] = useState<ConnectionLine[]>([]);

    useEffect(() => {
        const newConnections: ConnectionLine[] = [];

        products.forEach(product => {
            product.eventDatas.forEach(event => {
                if ('points' in event) {
                    event.points.forEach(point => {
                        if ('action' in point && point.action?.triggers) {
                            point.action.triggers.forEach(trigger => {
                                if (trigger.triggeredAsset) {
                                    const targetPoint = getPointByUuid(
                                        product.productId,
                                        trigger.triggeredAsset.triggeredModel.modelUuid,
                                        trigger.triggeredAsset.triggeredPoint.pointUuid
                                    );

                                    if (targetPoint) {
                                        const startPos = new THREE.Vector3(...point.position);
                                        const endPos = new THREE.Vector3(...targetPoint.position);
                                        const midPos = new THREE.Vector3()
                                            .addVectors(startPos, endPos)
                                            .multiplyScalar(0.5)
                                            .add(new THREE.Vector3(0, 2, 0));

                                        newConnections.push({
                                            id: `${point.uuid}-${targetPoint.uuid}-${trigger.triggerUuid}`,
                                            start: startPos,
                                            end: endPos,
                                            mid: midPos,
                                            trigger
                                        });
                                    }
                                }
                            });
                        }
                    });
                }
            });
        });

        setConnections(newConnections);
    }, [products]);

    useEffect(() => {
        console.log('connections: ', connections);
    }, connections)

    useEffect(() => {
        const canvasElement = gl.domElement;

        let drag = false;
        let isRightMouseDown = false;

        const onMouseDown = (evt: MouseEvent) => {
            if (selectedAsset) {
                clearSelectedAsset();
            }
            if (evt.button === 2) {
                isRightMouseDown = true;
                drag = false;
            }
        };

        const onMouseUp = (evt: MouseEvent) => {
            if (evt.button === 2) {
                isRightMouseDown = false;
            }
        }

        const onMouseMove = () => {
            if (isRightMouseDown) {
                drag = true;
            }
        };

        const handleRightClick = (evt: MouseEvent) => {
            if (drag) return;
            evt.preventDefault();

            const intersects = raycaster.intersectObjects(scene.children, true);
            if (intersects.length === 0) return;

            const currentObject = intersects[0].object;
            if (!currentObject || currentObject.name !== 'Event-Sphere') return;

            const modelUuid = currentObject.userData.modelUuid;
            const pointUuid = currentObject.userData.pointUuid;

            if (selectedProduct && getIsEventInProduct(selectedProduct.productId, modelUuid)) {

                const point = getPointByUuid(
                    selectedProduct.productId,
                    modelUuid,
                    pointUuid
                );

                if (!point) return;

                let actionUuid: string | undefined;
                if ('action' in point && point.action) {
                    actionUuid = point.action.actionUuid;
                } else if ('actions' in point && point.actions.length === 1) {
                    actionUuid = point.actions[0].actionUuid;
                }

                if (!firstSelectedPoint) {
                    setFirstSelectedPoint({
                        productId: selectedProduct.productId,
                        modelUuid,
                        pointUuid,
                        actionUuid
                    });
                } else {
                    const trigger: TriggerSchema = {
                        triggerUuid: THREE.MathUtils.generateUUID(),
                        triggerName: `Trigger ${firstSelectedPoint.pointUuid.slice(0, 4)} → ${pointUuid.slice(0, 4)}`,
                        triggerType: "onComplete",
                        delay: 0,
                        triggeredAsset: {
                            triggeredModel: {
                                modelName: currentObject.parent?.parent?.name || 'Unknown',
                                modelUuid: modelUuid
                            },
                            triggeredPoint: {
                                pointName: currentObject.name,
                                pointUuid: pointUuid
                            },
                            triggeredAction: actionUuid ? {
                                actionName: getActionByUuid(selectedProduct.productId, actionUuid)?.actionName || 'Action',
                                actionUuid: actionUuid
                            } : null
                        }
                    };

                    if (firstSelectedPoint.actionUuid) {
                        addTrigger(firstSelectedPoint.actionUuid, trigger);
                    }
                    setFirstSelectedPoint(null);
                }
            } else if (!getIsEventInProduct(selectedProduct.productId, modelUuid) && firstSelectedPoint) {
                handleAddEventToProduct({
                    event: useEventsStore.getState().getEventByModelUuid(modelUuid),
                    addEvent,
                    selectedProduct,
                })

                const point = getPointByUuid(
                    selectedProduct.productId,
                    modelUuid,
                    pointUuid
                );

                if (!point) return;

                let actionUuid: string | undefined;
                if ('action' in point && point.action) {
                    actionUuid = point.action.actionUuid;
                } else if ('actions' in point && point.actions.length === 1) {
                    actionUuid = point.actions[0].actionUuid;
                }

                const trigger: TriggerSchema = {
                    triggerUuid: THREE.MathUtils.generateUUID(),
                    triggerName: `Trigger ${firstSelectedPoint.pointUuid.slice(0, 4)} → ${pointUuid.slice(0, 4)}`,
                    triggerType: "onComplete",
                    delay: 0,
                    triggeredAsset: {
                        triggeredModel: {
                            modelName: currentObject.parent?.parent?.name || 'Unknown',
                            modelUuid: modelUuid
                        },
                        triggeredPoint: {
                            pointName: currentObject.name,
                            pointUuid: pointUuid
                        },
                        triggeredAction: actionUuid ? {
                            actionName: getActionByUuid(selectedProduct.productId, actionUuid)?.actionName || 'Action',
                            actionUuid: actionUuid
                        } : null
                    }
                };

                if (firstSelectedPoint.actionUuid) {
                    addTrigger(firstSelectedPoint.actionUuid, trigger);
                }
                setFirstSelectedPoint(null);
            }
        };

        if (subModule === 'simulations') {
            canvasElement.addEventListener("mousedown", onMouseDown);
            canvasElement.addEventListener("mouseup", onMouseUp);
            canvasElement.addEventListener("mousemove", onMouseMove);
            canvasElement.addEventListener('contextmenu', handleRightClick);
        }

        return () => {
            canvasElement.removeEventListener("mousedown", onMouseDown);
            canvasElement.removeEventListener("mouseup", onMouseUp);
            canvasElement.removeEventListener("mousemove", onMouseMove);
            canvasElement.removeEventListener('contextmenu', handleRightClick);
        };

    }, [gl, subModule, selectedProduct, firstSelectedPoint]);

    return (
        <>
        </>
    );
}

export default TriggerConnector;