import React, { useEffect, useRef, useState } from 'react';
import startPoint from "../../../../assets/gltf-glb/arrow_green.glb";
import startEnd from "../../../../assets/gltf-glb/arrow_red.glb";
import * as THREE from "three";
import { useGLTF } from '@react-three/drei';
import { useFrame, useThree } from '@react-three/fiber';
import { useSelectedEventSphere } from '../../../../store/simulation/useSimulationStore';
import { useVehicleStore } from '../../../../store/simulation/useVehicleStore';
import * as Types from "../../../../types/world/worldTypes";
const VehicleUI = () => {
    const { scene: startScene } = useGLTF(startPoint) as any;
    const { scene: endScene } = useGLTF(startEnd) as any;
    const startMarker = useRef<THREE.Group>(null);
    const endMarker = useRef<THREE.Group>(null);
    const prevMousePos = useRef({ x: 0, y: 0 });
    const { selectedEventSphere } = useSelectedEventSphere();
    const { vehicles, updateVehicle } = useVehicleStore();
    const [startPosition, setStartPosition] = useState<[number, number, number]>([0, 0, 0]);
    const [endPosition, setEndPosition] = useState<[number, number, number]>([0, 0, 0]);
    const [startRotation, setStartRotation] = useState<[number, number, number]>([0, 0, 0]);
    const [endRotation, setEndRotation] = useState<[number, number, number]>([0, 0, 0]);
    const [isDragging, setIsDragging] = useState<"start" | "end" | null>(null);
    const [isRotating, setIsRotating] = useState<"start" | "end" | null>(null);
    const { raycaster } = useThree();
    const plane = useRef(new THREE.Plane(new THREE.Vector3(0, 1, 0), 0));
    const state: Types.ThreeState = useThree();
    const controls: any = state.controls;

    useEffect(() => {
        if (!selectedEventSphere) return;
        const selectedVehicle = vehicles.find(
            (vehicle: any) => vehicle.modelUuid === selectedEventSphere.userData.modelUuid
        );

        if (selectedVehicle?.point?.action) {
            const { pickUpPoint, unLoadPoint } = selectedVehicle.point.action;

            if (pickUpPoint) {
                const pickupPosition = new THREE.Vector3(
                    pickUpPoint.position.x,
                    pickUpPoint.position.y,
                    pickUpPoint.position.z
                );
                const pickupRotation = new THREE.Vector3(
                    pickUpPoint.rotation.x,
                    pickUpPoint.rotation.y,
                    pickUpPoint.rotation.z
                );
                pickupPosition.y = 0; 
                setStartPosition([pickupPosition.x, 0, pickupPosition.z]);
                setStartRotation([pickupRotation.x, pickupRotation.y, pickupRotation.z]);
            } else {
                const defaultLocal = new THREE.Vector3(0, 0, 1.5);
                const defaultWorld = selectedEventSphere.localToWorld(defaultLocal);
                defaultWorld.y = 0; 
                setStartPosition([defaultWorld.x, 0, defaultWorld.z]);
                setStartRotation([0, 0, 0]);
            }

            if (unLoadPoint) {
                const unLoadPosition = new THREE.Vector3(
                    unLoadPoint.position.x,
                    unLoadPoint.position.y,
                    unLoadPoint.position.z
                );
                const unLoadRotation = new THREE.Vector3(
                    unLoadPoint.rotation.x,
                    unLoadPoint.rotation.y,
                    unLoadPoint.position.z
                );
                unLoadPosition.y = 0; // Force y to 0
                setEndPosition([unLoadPosition.x, 0, unLoadPosition.z]);
                setEndRotation([unLoadRotation.x, unLoadRotation.y, unLoadRotation.z]);
            } else {
                const defaultLocal = new THREE.Vector3(0, 0, -1.5);
                const defaultWorld = selectedEventSphere.localToWorld(defaultLocal);
                defaultWorld.y = 0; // Force y to 0
                setEndPosition([defaultWorld.x, 0, defaultWorld.z]);
                setEndRotation([0, 0, 0]);
            }
        }
    }, [selectedEventSphere]);

    useFrame(() => {
        if (!isDragging) return;
        const intersectPoint = new THREE.Vector3();
        const intersects = raycaster.ray.intersectPlane(plane.current, intersectPoint);

        if (intersects) {
            intersectPoint.y = 0; // Force y to 0
            if (isDragging === "start") {
                setStartPosition([intersectPoint.x, 0, intersectPoint.z]);
            }
            if (isDragging === "end") {
                setEndPosition([intersectPoint.x, 0, intersectPoint.z]);
            }
        }
    });

    useFrame((state) => {
        if (!isRotating) return;

        const currentPointerX = state.pointer.x;
        const deltaX = currentPointerX - prevMousePos.current.x;
        prevMousePos.current.x = currentPointerX;

        const marker = isRotating === "start" ? startMarker.current : endMarker.current;

        if (marker) {
            const rotationSpeed = 10;
            marker.rotation.y += deltaX * rotationSpeed;
            if (isRotating === 'start') {
                setStartRotation([marker.rotation.x, marker.rotation.y, marker.rotation.z])
            } else {

                setEndRotation([marker.rotation.x, marker.rotation.y, marker.rotation.z])
            }
        }
    });


    const handlePointerDown = (e: any, state: "start" | "end", rotation: "start" | "end") => {

        if (e.object.name === "handle") {
            const normalizedX = (e.clientX / window.innerWidth) * 2 - 1;
            const normalizedY = -(e.clientY / window.innerHeight) * 2 + 1;
            prevMousePos.current = { x: normalizedX, y: normalizedY };
            setIsRotating(rotation);
            if (controls) controls.enabled = false;
            setIsDragging(null);

        } else {
            setIsDragging(state);
            setIsRotating(null);
            if (controls) controls.enabled = false;
        }
    };

    const handlePointerUp = () => {
        controls.enabled = true;
        setIsDragging(null);
        setIsRotating(null);

        if (selectedEventSphere?.userData.modelUuid) {
            const updatedVehicle = vehicles.find(
                (vehicle) => vehicle.modelUuid === selectedEventSphere.userData.modelUuid
            );

            if (updatedVehicle) {
                updateVehicle(selectedEventSphere.userData.modelUuid, {
                    point: {
                        ...updatedVehicle.point,
                        action: {
                            ...updatedVehicle.point?.action,
                            pickUpPoint: {
                                position: {
                                    x: startPosition[0],
                                    y: startPosition[1],
                                    z: startPosition[2],
                                },
                                rotation: {
                                    x: startRotation[0],
                                    y: startRotation[1],
                                    z: startRotation[2],
                                },
                            },
                            unLoadPoint: {
                                position: {
                                    x: endPosition[0],
                                    y: endPosition[1],
                                    z: endPosition[2],
                                },
                                rotation: {
                                    x: endRotation[0],
                                    y: endRotation[1],
                                    z: endRotation[2],
                                },
                            },
                        },
                    },
                });
            }
        }
    };

    useEffect(() => {
        const handleGlobalPointerUp = () => {
            setIsDragging(null);
            setIsRotating(null);
            if (controls) controls.enabled = true;
            handlePointerUp();
        };

        if (isDragging || isRotating) {
            window.addEventListener("pointerup", handleGlobalPointerUp);
        }

        return () => {
            window.removeEventListener("pointerup", handleGlobalPointerUp);
        };
    }, [isDragging, isRotating, startPosition, startRotation, endPosition, endRotation]);

    return (
        startPosition.length > 0 && endPosition.length > 0 ? (
            <mesh>
                <primitive
                    name="start"
                    object={startScene}
                    ref={startMarker}
                    position={startPosition}
                    onPointerDown={(e: any) => {
                        e.stopPropagation();
                        handlePointerDown(e, "start", "start");
                    }}
                    onPointerMissed={() => {
                        controls.enabled = true;
                        setIsDragging(null);
                        setIsRotating(null);
                    }}
                />

                <primitive
                    name="end"
                    object={endScene}
                    ref={endMarker}
                    position={endPosition}
                    onPointerDown={(e: any) => {
                        e.stopPropagation();
                        handlePointerDown(e, "end", "end");
                    }}
                    onPointerMissed={() => {
                        controls.enabled = true;
                        setIsDragging(null);
                        setIsRotating(null);
                    }}
                />
            </mesh>
        ) : null
    );
}
export default VehicleUI;


